@@ -29,10 +29,10 @@
 #define HAVE_SYS_SYSCALL_H 1
 #define HAVE_FCNTL_H 1
 #define HAVE_SYS_FCNTL_H 1
+#define HAVE_SYS_PARAM_H 1
 #define HAVE_SYS_SELECT_H 1
 #define HAVE_SYS_TIME_H 1
 #define HAVE_SYS_TIMES_H 1
-#define HAVE_SYS_PARAM_H 1
 #define HAVE_PWD_H 1
 #define HAVE_GRP_H 1
 #define HAVE_A_OUT_H 1
@@ -44,8 +44,10 @@
 #define HAVE_LOCALE_H 1
 #define HAVE_TIME_H 1
 #define HAVE_SYS_SOCKET_H 1
+#define HAVE_SYS_UIO_H 1
 #define HAVE_GMP_H 1
 #define HAVE_LIBGMP 1
+#define HAVE_TYPEOF 1
 #define HAVE_LONG_LONG 1
 #define HAVE_OFF_T 1
 #define SIZEOF_INT 4
@@ -60,6 +62,8 @@
 #define SIZEOF_DOUBLE 8
 #define SIZEOF_TIME_T 8
 #define SIZEOF_CLOCK_T 8
+#define PACKED_STRUCT(x) x __attribute__((packed))
+#define PACKED_STRUCT_UNALIGNED(x) x
 #define PRI_LL_PREFIX "ll"
 #define rb_pid_t pid_t
 #define SIGNEDNESS_OF_PID_T -1
@@ -131,8 +135,9 @@
 #define HAVE_ST_BLOCKS 1
 #define HAVE_STRUCT_STAT_ST_RDEV 1
 #define HAVE_ST_RDEV 1
-#define SIZEOF_STRUCT_STAT_ST_SIZE SIZEOF_LONG
-#define SIZEOF_STRUCT_STAT_ST_BLOCKS SIZEOF_LONG
+#define SIZEOF_STRUCT_STAT_ST_SIZE SIZEOF_OFF_T
+#define SIZEOF_STRUCT_STAT_ST_BLOCKS SIZEOF_OFF_T
+#define SIZEOF_STRUCT_STAT_ST_INO SIZEOF_LONG
 #define HAVE_STRUCT_STAT_ST_ATIM 1
 #define HAVE_STRUCT_STAT_ST_ATIMESPEC 1
 #define HAVE_STRUCT_STAT_ST_ATIMENSEC 1
@@ -174,36 +179,45 @@
 #define GETGROUPS_T gid_t
 #define RETSIGTYPE void
 #define HAVE_ALLOCA 1
-#define HAVE_DUP2 1
-#define HAVE_MEMMOVE 1
-#define HAVE_STRERROR 1
-#define HAVE_STRCHR 1
-#define HAVE_STRSTR 1
-#define HAVE_CRYPT 1
-#define HAVE_FLOCK 1
-#define HAVE_ISNAN 1
-#define HAVE_FINITE 1
-#define HAVE_ISINF 1
-#define HAVE_HYPOT 1
 #define HAVE_ACOSH 1
-#define HAVE_ERF 1
-#define HAVE_TGAMMA 1
-#define HAVE_LGAMMA_R 1
 #define HAVE_CBRT 1
-#define HAVE_STRLCPY 1
-#define HAVE_STRLCAT 1
+#define HAVE_CRYPT 1
+#define HAVE_DUP2 1
+#define HAVE_ERF 1
 #define HAVE_FFS 1
+#define HAVE_FINITE 1
+#define HAVE_FLOCK 1
+#define HAVE_HYPOT 1
+#define HAVE_ISINF 1
+#define HAVE_ISNAN 1
+#define HAVE_LGAMMA_R 1
+#define HAVE_MEMMOVE 1
+#define HAVE_NEXTAFTER 1
 #define HAVE_SETPROCTITLE 1
+#define HAVE_STRCHR 1
+#define HAVE_STRERROR 1
+#define HAVE_STRLCAT 1
+#define HAVE_STRLCPY 1
+#define HAVE_STRSTR 1
+#define HAVE_TGAMMA 1
 #define HAVE_SIGNBIT 1
+#define HAVE_FORK 1
+#define HAVE_VFORK 1
+#define HAVE_WORKING_VFORK 1
+#define HAVE_WORKING_FORK 1
 #define HAVE___SYSCALL 1
 #define HAVE__LONGJMP 1
 #define HAVE__SETJMP 1
+#define HAVE_ATAN2L 1
+#define HAVE_ATAN2F 1
 #define HAVE_CHROOT 1
 #define HAVE_CLOCK_GETTIME 1
 #define HAVE_COSH 1
 #define HAVE_DAEMON 1
+#define HAVE_DIRFD 1
 #define HAVE_DL_ITERATE_PHDR 1
 #define HAVE_DLOPEN 1
+#define HAVE_DLADDR 1
 #define HAVE_DUP 1
 #define HAVE_DUP3 1
 #define HAVE_ENDGRENT 1
@@ -212,16 +226,18 @@
 #define HAVE_FCNTL 1
 #define HAVE_FDATASYNC 1
 #define HAVE_FMOD 1
-#define HAVE_FORK 1
 #define HAVE_FSYNC 1
 #define HAVE_FTRUNCATE 1
 #define HAVE_GETCWD 1
+#define HAVE_GETGRNAM 1
 #define HAVE_GETGRNAM_R 1
 #define HAVE_GETGROUPS 1
 #define HAVE_GETPGID 1
 #define HAVE_GETPGRP 1
 #define HAVE_GETPRIORITY 1
 #define HAVE_GETPWNAM_R 1
+#define HAVE_GETRESGID 1
+#define HAVE_GETRESUID 1
 #define HAVE_GETRLIMIT 1
 #define HAVE_GETSID 1
 #define HAVE_GETTIMEOFDAY 1
@@ -237,7 +253,9 @@
 #define HAVE_LOG2 1
 #define HAVE_LSTAT 1
 #define HAVE_MBLEN 1
+#define HAVE_WRITEV 1
 #define HAVE_MEMRCHR 1
+#define HAVE_MEMMEM 1
 #define HAVE_MKTIME 1
 #define HAVE_PIPE2 1
 #define HAVE_POLL 1
@@ -284,6 +302,7 @@
 #define HAVE_BUILTIN___BUILTIN_CLZLL 1
 #define HAVE_BUILTIN___BUILTIN_CHOOSE_EXPR 1
 #define HAVE_BUILTIN___BUILTIN_TYPES_COMPATIBLE_P 1
+#define ATAN2_INF_C99 1
 #define HAVE_CLOCK_GETRES 1
 #define HAVE_STRUCT_TM_TM_ZONE 1
 #define HAVE_TM_ZONE 1
@@ -296,9 +315,6 @@
 #define NEGATIVE_TIME_T 1
 #define POSIX_SIGNAL 1
 #define RSHIFT(x,y) ((x)>>(int)(y))
-#define FILE_COUNT _r
-#define FILE_READPTR _p
-#define SIZEOF_STRUCT_STAT_ST_INO SIZEOF_LONG
 #define HAVE__SC_CLK_TCK 1
 #define STACK_GROW_DIRECTION -1
 #define _REENTRANT 1
@@ -323,6 +339,5 @@
 #define RUBY_LONGJMP(env,val) _longjmp((env),val)
 #define RUBY_JMP_BUF jmp_buf
 #define HAVE_PTHREAD_H 1
-#define HAVE_MEMMEM 1
 #define RUBY_PLATFORM "mips64el-openbsd"
 #endif /* INCLUDE_RUBY_CONFIG_H */
