@@ -1,35 +1,65 @@
 #
 #   forwardable.rb -
-#   	$Release Version: 1.1$
-#   	$Revision: 31833 $
-#   	by Keiju ISHITSUKA(keiju@ishitsuka.com)
-#	original definition by delegator.rb
+#       $Release Version: 1.1$
+#       $Revision: 40906 $
+#       by Keiju ISHITSUKA(keiju@ishitsuka.com)
+#       original definition by delegator.rb
 #       Revised by Daniel J. Berger with suggestions from Florian Gross.
 #
 #       Documentation by James Edward Gray II and Gavin Sinclair
+
+
+
+# The Forwardable module provides delegation of specified
+# methods to a designated object, using the methods #def_delegator
+# and #def_delegators.
 #
-# == Introduction
+# For example, say you have a class RecordCollection which
+# contains an array <tt>@records</tt>.  You could provide the lookup method
+# #record_number(), which simply calls #[] on the <tt>@records</tt>
+# array, like this:
 #
-# This library allows you delegate method calls to an object, on a method by
-# method basis.
+#   require 'forwardable'
 #
-# == Notes
+#   class RecordCollection
+#     attr_accessor :records
+#     extend Forwardable
+#     def_delegator :@records, :[], :record_number
+#   end
 #
-# Be advised, RDoc will not detect delegated methods.
+# We can use the lookup method like so:
 #
-# <b>forwardable.rb provides single-method delegation via the
-# def_delegator() and def_delegators() methods.  For full-class
-# delegation via DelegateClass(), see delegate.rb.</b>
+#   r = RecordCollection.new
+#   r.records = [4,5,6]
+#   r.record_number(0)  # => 4
 #
-# == Examples
+# Further, if you wish to provide the methods #size, #<<, and #map,
+# all of which delegate to @records, this is how you can do it:
 #
-# === Forwardable
+#   class RecordCollection # re-open RecordCollection class
+#     def_delegators :@records, :size, :<<, :map
+#   end
 #
-# Forwardable makes building a new class based on existing work, with a proper
-# interface, almost trivial.  We want to rely on what has come before obviously,
-# but with delegation we can take just the methods we need and even rename them
-# as appropriate.  In many cases this is preferable to inheritance, which gives
-# us the entire old interface, even if much of it isn't needed.
+#   r = RecordCollection.new
+#   r.records = [1,2,3]
+#   r.record_number(0)   # => 1
+#   r.size               # => 3
+#   r << 4               # => [1, 2, 3, 4]
+#   r.map { |x| x * 2 }  # => [2, 4, 6, 8]
+#
+# You can even extend regular objects with Forwardable.
+#
+#   my_hash = Hash.new
+#   my_hash.extend Forwardable              # prepare object for delegation
+#   my_hash.def_delegator "STDOUT", "puts"  # add delegation for STDOUT.puts()
+#   my_hash.puts "Howdy!"
+#
+# == Another example
+#
+# We want to rely on what has come before obviously, but with delegation we can
+# take just the methods we need and even rename them as appropriate.  In many
+# cases this is preferable to inheritance, which gives us the entire old
+# interface, even if much of it isn't needed.
 #
 #   class Queue
 #     extend Forwardable
@@ -60,7 +90,7 @@
 #   q.clear
 #   puts q.first
 #
-# <i>Prints:</i>
+# This should output:
 #
 #   2
 #   3
@@ -70,72 +100,24 @@
 #   Ruby
 #   nil
 #
-# SingleForwardable can be used to setup delegation at the object level as well.
+# == Notes
 #
-#    printer = String.new
-#    printer.extend SingleForwardable        # prepare object for delegation
-#    printer.def_delegator "STDOUT", "puts"  # add delegation for STDOUT.puts()
-#    printer.puts "Howdy!"
+# Be advised, RDoc will not detect delegated methods.
 #
-# Also, SingleForwardable can be use to Class or Module.
+# +forwardable.rb+ provides single-method delegation via the def_delegator and
+# def_delegators methods. For full-class delegation via DelegateClass, see
+# +delegate.rb+.
 #
-#    module Facade
-#      extend SingleForwardable
-#      def_delegator :Implementation, :service
-#
-#      class Implementation
-#	  def service...
-#      end
-#    end
-#
-# If you want to use both Forwardable and SingleForwardable, you can
-# use methods def_instance_delegator and def_single_delegator, etc.
-#
-# If the object isn't a Module and Class, You can too extend
-# Forwardable module.
-#    printer = String.new
-#    printer.extend Forwardable              # prepare object for delegation
-#    printer.def_delegator "STDOUT", "puts"  # add delegation for STDOUT.puts()
-#    printer.puts "Howdy!"
-#
-# <i>Prints:</i>
-#
-#    Howdy!
-
-#
-# The Forwardable module provides delegation of specified
-# methods to a designated object, using the methods #def_delegator
-# and #def_delegators.
-#
-# For example, say you have a class RecordCollection which
-# contains an array <tt>@records</tt>.  You could provide the lookup method
-# #record_number(), which simply calls #[] on the <tt>@records</tt>
-# array, like this:
-#
-#   class RecordCollection
-#     extend Forwardable
-#     def_delegator :@records, :[], :record_number
-#   end
-#
-# Further, if you wish to provide the methods #size, #<<, and #map,
-# all of which delegate to @records, this is how you can do it:
-#
-#   class RecordCollection
-#     # extend Forwardable, but we did that above
-#     def_delegators :@records, :size, :<<, :map
-#   end
-#   f = Foo.new
-#   f.printf ...
-#   f.gets
-#   f.content_at(1)
-#
-# Also see the example at forwardable.rb.
-
 module Forwardable
+  # Version of +forwardable.rb+
   FORWARDABLE_VERSION = "1.1.0"
 
+  FILE_REGEXP = %r"#{Regexp.quote(__FILE__)}"
+
   @debug = nil
   class << self
+    # If true, <tt>__FILE__</tt> will remain in the backtrace in the event an
+    # Exception is raised.
     attr_accessor :debug
   end
 
@@ -175,15 +157,34 @@ module Forwardable
     end
   end
 
+  # Define +method+ as delegator instance method with an optional
+  # alias name +ali+. Method calls to +ali+ will be delegated to
+  # +accessor.method+.
+  #
+  #   class MyQueue
+  #     extend Forwardable
+  #     attr_reader :queue
+  #     def initialize
+  #       @queue = []
+  #     end
+  #
+  #     def_delegator :@queue, :push, :mypush
+  #   end
+  #
+  #   q = MyQueue.new
+  #   q.mypush 42
+  #   q.queue    #=> [42]
+  #   q.push 23  #=> NoMethodError
+  #
   def def_instance_delegator(accessor, method, ali = method)
     line_no = __LINE__; str = %{
       def #{ali}(*args, &block)
-	begin
-	  #{accessor}.__send__(:#{method}, *args, &block)
-	rescue Exception
-	  $@.delete_if{|s| %r"#{Regexp.quote(__FILE__)}"o =~ s} unless Forwardable::debug
-	  ::Kernel::raise
-	end
+        begin
+          #{accessor}.__send__(:#{method}, *args, &block)
+        rescue Exception
+          $@.delete_if{|s| Forwardable::FILE_REGEXP =~ s} unless Forwardable::debug
+          ::Kernel::raise
+        end
       end
     }
     # If it's not a class or module, it's an instance
@@ -200,9 +201,30 @@ module Forwardable
   alias def_delegator def_instance_delegator
 end
 
+# SingleForwardable can be used to setup delegation at the object level as well.
 #
-# Usage of The SingleForwardable is like Fowadable module.
+#    printer = String.new
+#    printer.extend SingleForwardable        # prepare object for delegation
+#    printer.def_delegator "STDOUT", "puts"  # add delegation for STDOUT.puts()
+#    printer.puts "Howdy!"
 #
+# Also, SingleForwardable can be used to set up delegation for a Class or Module.
+#
+#   class Implementation
+#     def self.service
+#       puts "serviced!"
+#     end
+#   end
+#
+#   module Facade
+#     extend SingleForwardable
+#     def_delegator :Implementation, :service
+#   end
+#
+#   Facade.service #=> serviced!
+#
+# If you want to use both Forwardable and SingleForwardable, you can
+# use methods def_instance_delegator and def_single_delegator, etc.
 module SingleForwardable
   # Takes a hash as its argument.  The key is a symbol or an array of
   # symbols.  These symbols correspond to method names.  The value is
@@ -240,20 +262,21 @@ module SingleForwardable
     end
   end
 
+  # :call-seq:
+  #   def_single_delegator(accessor, method, new_name=method)
   #
-  # Defines a method _method_ which delegates to _obj_ (i.e. it calls
-  # the method of the same name in _obj_).  If _new_name_ is
+  # Defines a method _method_ which delegates to _accessor_ (i.e. it calls
+  # the method of the same name in _accessor_).  If _new_name_ is
   # provided, it is used as the name for the delegate method.
-  #
   def def_single_delegator(accessor, method, ali = method)
-    line_no = __LINE__; str = %{
+    str = %{
       def #{ali}(*args, &block)
-	begin
-	  #{accessor}.__send__(:#{method}, *args, &block)
-	rescue Exception
-	  $@.delete_if{|s| %r"#{Regexp.quote(__FILE__)}"o =~ s} unless Forwardable::debug
-	  ::Kernel::raise
-	end
+        begin
+          #{accessor}.__send__(:#{method}, *args, &block)
+        rescue Exception
+          $@.delete_if{|s| Forwardable::FILE_REGEXP =~ s} unless Forwardable::debug
+          ::Kernel::raise
+        end
       end
     }
 
@@ -264,7 +287,3 @@ module SingleForwardable
   alias def_delegators def_single_delegators
   alias def_delegator def_single_delegator
 end
-
-
-
-
